@@ -1,10 +1,10 @@
 /*
-  Copyright (c) 2005, 2014, Oracle and/or its affiliates. All rights reserved.
+  Copyright (c) 2005, 2016, Oracle and/or its affiliates. All rights reserved.
 
   The MySQL Connector/J is licensed under the terms of the GPLv2
   <http://www.gnu.org/licenses/old-licenses/gpl-2.0.html>, like most MySQL Connectors.
   There are special exceptions to the terms and conditions of the GPLv2 as it is applied to
-  this software, see the FLOSS License Exception
+  this software, see the FOSS License Exception
   <http://www.mysql.com/about/legal/licensing/foss-exception.html>.
 
   This program is free software; you can redistribute it and/or modify it under the terms
@@ -31,11 +31,11 @@ import java.util.Iterator;
 import java.util.Map;
 import java.util.Properties;
 
-import testsuite.BaseTestCase;
-
 import com.mysql.jdbc.CharsetMapping;
 import com.mysql.jdbc.ConnectionImpl;
 
+import testsuite.BaseTestCase;
+
 public class ResultSetTest extends BaseTestCase {
 
     public ResultSetTest(String name) {
@@ -80,11 +80,11 @@ public class ResultSetTest extends BaseTestCase {
         c.close();
 
         Iterator<String> charsetNames = charsetsMap.keySet().iterator();
-        StringBuffer columns = new StringBuffer();
-        StringBuffer emptyBuf = new StringBuffer();
-        StringBuffer abcBuf = new StringBuffer();
-        StringBuffer repeatBuf = new StringBuffer();
-        StringBuffer selectBuf = new StringBuffer();
+        StringBuilder columns = new StringBuilder();
+        StringBuilder emptyBuf = new StringBuilder();
+        StringBuilder abcBuf = new StringBuilder();
+        StringBuilder repeatBuf = new StringBuilder();
+        StringBuilder selectBuf = new StringBuilder();
 
         int counter = 0;
 
@@ -129,8 +129,8 @@ public class ResultSetTest extends BaseTestCase {
 
         createTable("testPadding", "(" + columns.toString() + ", ord INT)");
 
-        this.stmt.executeUpdate("INSERT INTO testPadding VALUES (" + emptyBuf.toString() + ", 1), (" + abcBuf.toString() + ", 2), (" + repeatBuf.toString()
-                + ", 3)");
+        this.stmt.executeUpdate(
+                "INSERT INTO testPadding VALUES (" + emptyBuf.toString() + ", 1), (" + abcBuf.toString() + ", 2), (" + repeatBuf.toString() + ", 3)");
 
         try {
             Properties props = new Properties();
@@ -152,7 +152,7 @@ public class ResultSetTest extends BaseTestCase {
         }
     }
 
-    private void testPaddingForConnection(Connection paddedConn, int numChars, StringBuffer selectBuf) throws SQLException {
+    private void testPaddingForConnection(Connection paddedConn, int numChars, StringBuilder selectBuf) throws SQLException {
 
         String query = "SELECT " + selectBuf.toString() + " FROM testPadding ORDER by ord";
 
@@ -163,8 +163,8 @@ public class ResultSetTest extends BaseTestCase {
             for (int i = 0; i < numCols; i++) {
                 assertEquals(
                         "For column '" + this.rs.getMetaData().getColumnName(i + 1) + "' of collation "
-                                + ((com.mysql.jdbc.ResultSetMetaData) this.rs.getMetaData()).getColumnCharacterSet(i + 1), numChars, this.rs.getString(i + 1)
-                                .length());
+                                + ((com.mysql.jdbc.ResultSetMetaData) this.rs.getMetaData()).getColumnCharacterSet(i + 1),
+                        numChars, this.rs.getString(i + 1).length());
             }
         }
 
@@ -174,8 +174,8 @@ public class ResultSetTest extends BaseTestCase {
             for (int i = 0; i < numCols; i++) {
                 assertEquals(
                         "For column '" + this.rs.getMetaData().getColumnName(i + 1) + "' of collation "
-                                + ((com.mysql.jdbc.ResultSetMetaData) this.rs.getMetaData()).getColumnCharacterSet(i + 1), numChars, this.rs.getString(i + 1)
-                                .length());
+                                + ((com.mysql.jdbc.ResultSetMetaData) this.rs.getMetaData()).getColumnCharacterSet(i + 1),
+                        numChars, this.rs.getString(i + 1).length());
             }
         }
 
@@ -186,8 +186,8 @@ public class ResultSetTest extends BaseTestCase {
                 for (int i = 0; i < numCols; i++) {
                     assertEquals(
                             "For column '" + this.rs.getMetaData().getColumnName(i + 1) + "' of collation "
-                                    + ((com.mysql.jdbc.ResultSetMetaData) this.rs.getMetaData()).getColumnCharacterSet(i + 1), numChars,
-                            this.rs.getString(i + 1).length());
+                                    + ((com.mysql.jdbc.ResultSetMetaData) this.rs.getMetaData()).getColumnCharacterSet(i + 1),
+                            numChars, this.rs.getString(i + 1).length());
                 }
             }
         }
@@ -204,8 +204,8 @@ public class ResultSetTest extends BaseTestCase {
                 } else {
                     assertEquals(
                             "For column '" + this.rs.getMetaData().getColumnName(i + 1) + "' of collation "
-                                    + ((com.mysql.jdbc.ResultSetMetaData) this.rs.getMetaData()).getColumnCharacterSet(i + 1), numChars,
-                            this.rs.getString(i + 1).length());
+                                    + ((com.mysql.jdbc.ResultSetMetaData) this.rs.getMetaData()).getColumnCharacterSet(i + 1),
+                            numChars, this.rs.getString(i + 1).length());
                 }
             }
         }
@@ -222,8 +222,8 @@ public class ResultSetTest extends BaseTestCase {
                 } else {
                     assertEquals(
                             "For column '" + this.rs.getMetaData().getColumnName(i + 1) + "' of collation "
-                                    + ((com.mysql.jdbc.ResultSetMetaData) this.rs.getMetaData()).getColumnCharacterSet(i + 1), numChars,
-                            this.rs.getString(i + 1).length());
+                                    + ((com.mysql.jdbc.ResultSetMetaData) this.rs.getMetaData()).getColumnCharacterSet(i + 1),
+                            numChars, this.rs.getString(i + 1).length());
                 }
             }
         }
@@ -234,13 +234,15 @@ public class ResultSetTest extends BaseTestCase {
             while (this.rs.next()) {
                 for (int i = 0; i < numCols; i++) {
                     if (this.rs.getRow() != 3) {
-                        assertTrue("For column '" + this.rs.getMetaData().getColumnName(i + 1) + "' of collation "
-                                + ((com.mysql.jdbc.ResultSetMetaData) this.rs.getMetaData()).getColumnCharacterSet(i + 1), numChars != this.rs.getString(i + 1)
-                                .length());
+                        assertTrue(
+                                "For column '" + this.rs.getMetaData().getColumnName(i + 1) + "' of collation "
+                                        + ((com.mysql.jdbc.ResultSetMetaData) this.rs.getMetaData()).getColumnCharacterSet(i + 1),
+                                numChars != this.rs.getString(i + 1).length());
                     } else {
-                        assertEquals("For column '" + this.rs.getMetaData().getColumnName(i + 1) + "' of collation "
-                                + ((com.mysql.jdbc.ResultSetMetaData) this.rs.getMetaData()).getColumnCharacterSet(i + 1), numChars, this.rs.getString(i + 1)
-                                .length());
+                        assertEquals(
+                                "For column '" + this.rs.getMetaData().getColumnName(i + 1) + "' of collation "
+                                        + ((com.mysql.jdbc.ResultSetMetaData) this.rs.getMetaData()).getColumnCharacterSet(i + 1),
+                                numChars, this.rs.getString(i + 1).length());
                     }
                 }
             }
